@@ -3,7 +3,8 @@ Reimplementations of constructs introduced in later versions of Python than
 we support. Also some functions that are needed SymPy-wide and are located
 here for easy import.
 """
-from __future__ import print_function, division
+
+from typing import Tuple, Type
 
 import operator
 from collections import defaultdict
@@ -15,20 +16,8 @@ Python 2 and Python 3 compatible imports
 String and Unicode compatible changes:
     * `unicode()` removed in Python 3, import `unicode` for Python 2/3
       compatible function
-    * `unichr()` removed in Python 3, import `unichr` for Python 2/3 compatible
-      function
     * Use `u()` for escaped unicode sequences (e.g. u'\u2020' -> u('\u2020'))
     * Use `u_decode()` to decode utf-8 formatted unicode strings
-    * `string_types` gives str in Python 3, unicode and str in Python 2,
-      equivalent to basestring
-
-Integer related changes:
-    * `long()` removed in Python 3, import `long` for Python 2/3 compatible
-      function
-    * `integer_types` gives int in Python 3, int and long in Python 2
-
-Types related changes:
-    * `class_types` gives type in Python 3, type and ClassType in Python 2
 
 Renamed function attributes:
     * Python 2 `.func_code`, Python 3 `.__func__`, access with
@@ -42,11 +31,7 @@ Moved modules:
     * `reduce()`
     * `StringIO()`
     * `cStringIO()` (same as `StingIO()` in Python 3)
-    * Python 2 `__builtins__`, access with Python 3 name, `builtins`
-
-Iterator/list changes:
-    * `xrange` renamed as `range` in Python 3, import `range` for Python 2/3
-      compatible iterator version of range.
+    * Python 2 `__builtin__`, access with Python 3 name, `builtins`
 
 exec:
     * Use `exec_()`, with parameters `exec_(code, globs=None, locs=None)`
@@ -61,121 +46,46 @@ Metaclasses:
                 pass
 """
 
+__all__ = [
+    'PY3', 'int_info', 'SYMPY_INTS', 'clock',
+    'unicode', 'u_decode', 'get_function_code', 'gmpy',
+    'get_function_globals', 'get_function_name', 'builtins', 'reduce',
+    'StringIO', 'cStringIO', 'exec_', 'Mapping', 'Callable',
+    'MutableMapping', 'MutableSet', 'Iterable', 'Hashable', 'unwrap',
+    'accumulate', 'with_metaclass', 'NotIterable', 'iterable', 'is_sequence',
+    'as_int', 'default_sort_key', 'ordered', 'GROUND_TYPES', 'HAS_GMPY',
+]
+
 import sys
-PY3 = sys.version_info[0] > 2
+PY3 = True
 
-if PY3:
-    class_types = type,
-    integer_types = (int,)
-    string_types = (str,)
-    long = int
-    int_info = sys.int_info
+int_info = sys.int_info
 
-    # String / unicode compatibility
-    unicode = str
-    unichr = chr
+# String / unicode compatibility
+unicode = str
 
-    def u_decode(x):
-        return x
+def u_decode(x):
+    return x
 
-    Iterator = object
+# Moved definitions
+get_function_code = operator.attrgetter("__code__")
+get_function_globals = operator.attrgetter("__globals__")
+get_function_name = operator.attrgetter("__name__")
 
-    # Moved definitions
-    get_function_code = operator.attrgetter("__code__")
-    get_function_globals = operator.attrgetter("__globals__")
-    get_function_name = operator.attrgetter("__name__")
+import builtins
+from functools import reduce
+from io import StringIO
+cStringIO = StringIO
 
-    import builtins
-    from functools import reduce
-    from io import StringIO
-    cStringIO = StringIO
+exec_ = getattr(builtins, "exec")
 
-    exec_=getattr(builtins, "exec")
+from collections.abc import (Mapping, Callable, MutableMapping,
+    MutableSet, Iterable, Hashable)
 
-    range=range
+from inspect import unwrap
+from itertools import accumulate
 
-    from collections.abc import (Mapping, Callable, MutableMapping,
-        MutableSet, Iterable, Hashable)
 
-    from inspect import unwrap
-else:
-    import codecs
-    import types
-
-    class_types = (type, types.ClassType)
-    integer_types = (int, long)
-    string_types = (str, unicode)
-    long = long
-    int_info = sys.long_info
-
-    # String / unicode compatibility
-    unicode = unicode
-    unichr = unichr
-
-    def u_decode(x):
-        return x.decode('utf-8')
-
-    class Iterator(object):
-        def next(self):
-            return type(self).__next__(self)
-
-    # Moved definitions
-    get_function_code = operator.attrgetter("func_code")
-    get_function_globals = operator.attrgetter("func_globals")
-    get_function_name = operator.attrgetter("func_name")
-
-    import __builtin__ as builtins
-    reduce = reduce
-    from StringIO import StringIO
-    from cStringIO import StringIO as cStringIO
-
-    def exec_(_code_, _globs_=None, _locs_=None):
-        """Execute code in a namespace."""
-        if _globs_ is None:
-            frame = sys._getframe(1)
-            _globs_ = frame.f_globals
-            if _locs_ is None:
-                _locs_ = frame.f_locals
-            del frame
-        elif _locs_ is None:
-            _locs_ = _globs_
-        exec("exec _code_ in _globs_, _locs_")
-    range=xrange
-
-    from collections import (Mapping, Callable, MutableMapping,
-        MutableSet, Iterable, Hashable)
-
-    def unwrap(func, stop=None):
-        """Get the object wrapped by *func*.
-
-       Follows the chain of :attr:`__wrapped__` attributes returning the last
-       object in the chain.
-
-       *stop* is an optional callback accepting an object in the wrapper chain
-       as its sole argument that allows the unwrapping to be terminated early if
-       the callback returns a true value. If the callback never returns a true
-       value, the last object in the chain is returned as usual. For example,
-       :func:`signature` uses this to stop unwrapping if any object in the
-       chain has a ``__signature__`` attribute defined.
-
-       :exc:`ValueError` is raised if a cycle is encountered.
-
-        """
-        if stop is None:
-            def _is_wrapper(f):
-                return hasattr(f, '__wrapped__')
-        else:
-            def _is_wrapper(f):
-                return hasattr(f, '__wrapped__') and not stop(f)
-        f = func  # remember the original func for error reporting
-        memo = {id(f)} # Memoise by id to tolerate non-hashable objects
-        while _is_wrapper(func):
-            func = func.__wrapped__
-            id_func = id(func)
-            if id_func in memo:
-                raise ValueError('wrapper loop when unwrapping {!r}'.format(f))
-            memo.add(id_func)
-        return func
 
 def with_metaclass(meta, *bases):
     """
@@ -207,9 +117,9 @@ def with_metaclass(meta, *bases):
     may omit it.
 
     >>> MyClass.__mro__
-    (<class 'MyClass'>, <... 'object'>)
+    (<class '...MyClass'>, <... 'object'>)
     >>> type(MyClass)
-    <class 'Meta'>
+    <class '...Meta'>
 
     """
     # This requires a bit of explanation: the basic idea is to make a dummy
@@ -229,16 +139,17 @@ def with_metaclass(meta, *bases):
 
 class NotIterable:
     """
-    Use this as mixin when creating a class which is not supposed to return
-    true when iterable() is called on its instances. I.e. avoid infinite loop
-    when calling e.g. list() on the instance
+    Use this as mixin when creating a class which is not supposed to
+    return true when iterable() is called on its instances because
+    calling list() on the instance, for example, would result in
+    an infinite loop.
     """
     pass
 
-def iterable(i, exclude=(string_types, dict, NotIterable)):
+def iterable(i, exclude=(str, dict, NotIterable)):
     """
     Return a boolean indicating whether ``i`` is SymPy iterable.
-    True also indicates that the iterator is finite, i.e. you e.g.
+    True also indicates that the iterator is finite, e.g. you can
     call list(...) on the instance.
 
     When SymPy is working with iterables, it is almost always assuming
@@ -332,50 +243,72 @@ def is_sequence(i, include=None):
             bool(include) and
             isinstance(i, include))
 
-try:
-    from itertools import zip_longest
-except ImportError:  # Python 2.7
-    from itertools import izip_longest as zip_longest
 
-
-try:
-    # Python 2.7
-    from string import maketrans
-except ImportError:
-    maketrans = str.maketrans
-
-
-def as_int(n):
+def as_int(n, strict=True):
     """
     Convert the argument to a builtin integer.
 
     The return value is guaranteed to be equal to the input. ValueError is
-    raised if the input has a non-integral value.
+    raised if the input has a non-integral value. When ``strict`` is True, this
+    uses `__index__ <https://docs.python.org/3/reference/datamodel.html#object.__index__>`_
+    and when it is False it uses ``int``.
+
 
     Examples
     ========
 
     >>> from sympy.core.compatibility import as_int
-    >>> from sympy import sqrt
-    >>> 3.0
-    3.0
-    >>> as_int(3.0) # convert to int and test for equality
+    >>> from sympy import sqrt, S
+
+    The function is primarily concerned with sanitizing input for
+    functions that need to work with builtin integers, so anything that
+    is unambiguously an integer should be returned as an int:
+
+    >>> as_int(S(3))
     3
-    >>> int(sqrt(10))
-    3
-    >>> as_int(sqrt(10))
+
+    Floats, being of limited precision, are not assumed to be exact and
+    will raise an error unless the ``strict`` flag is False. This
+    precision issue becomes apparent for large floating point numbers:
+
+    >>> big = 1e23
+    >>> type(big) is float
+    True
+    >>> big == int(big)
+    True
+    >>> as_int(big)
     Traceback (most recent call last):
     ...
     ValueError: ... is not an integer
+    >>> as_int(big, strict=False)
+    99999999999999991611392
 
+    Input that might be a complex representation of an integer value is
+    also rejected by default:
+
+    >>> one = sqrt(3 + 2*sqrt(2)) - sqrt(2)
+    >>> int(one) == 1
+    True
+    >>> as_int(one)
+    Traceback (most recent call last):
+    ...
+    ValueError: ... is not an integer
     """
-    try:
-        result = int(n)
-        if result != n:
-            raise TypeError
-    except TypeError:
-        raise ValueError('%s is not an integer' % (n,))
-    return result
+    if strict:
+        try:
+            if type(n) is bool:
+                raise TypeError
+            return operator.index(n)
+        except TypeError:
+            raise ValueError('%s is not an integer' % (n,))
+    else:
+        try:
+            result = int(n)
+        except TypeError:
+            raise ValueError('%s is not an integer' % (n,))
+        if n != result:
+            raise ValueError('%s is not an integer' % (n,))
+        return result
 
 
 def default_sort_key(item, order=None):
@@ -501,7 +434,7 @@ def default_sort_key(item, order=None):
     if isinstance(item, Basic):
         return item.sort_key(order=order)
 
-    if iterable(item, exclude=string_types):
+    if iterable(item, exclude=str):
         if isinstance(item, dict):
             args = item.items()
             unordered = True
@@ -521,9 +454,9 @@ def default_sort_key(item, order=None):
 
         cls_index, args = 10, (len(args), tuple(args))
     else:
-        if not isinstance(item, string_types):
+        if not isinstance(item, str):
             try:
-                item = sympify(item)
+                item = sympify(item, strict=True)
             except SympifyError:
                 # e.g. lambda x: x
                 pass
@@ -548,8 +481,11 @@ def _nodes(e):
     for which the sum of nodes is returned).
     """
     from .basic import Basic
+    from .function import Derivative
 
     if isinstance(e, Basic):
+        if isinstance(e, Derivative):
+            return _nodes(e.expr) + len(e.variables)
         return e.count(Basic)
     elif iterable(e):
         return 1 + sum(_nodes(ei) for ei in e)
@@ -565,9 +501,9 @@ def ordered(seq, keys=None, default=True, warn=False):
     then no other keys will be computed.
 
     Two default keys will be applied if 1) keys are not provided or 2) the
-    given keys don't resolve all ties (but only if `default` is True). The
-    two keys are `_nodes` (which places smaller expressions before large) and
-    `default_sort_key` which (if the `sort_key` for an object is defined
+    given keys don't resolve all ties (but only if ``default`` is True). The
+    two keys are ``_nodes`` (which places smaller expressions before large) and
+    ``default_sort_key`` which (if the ``sort_key`` for an object is defined
     properly) should resolve any ties.
 
     If ``warn`` is True then an error will be raised if there were no
@@ -629,7 +565,7 @@ def ordered(seq, keys=None, default=True, warn=False):
 
     This function is best used in cases when use of the first key is
     expected to be a good hashing function; if there are no unique hashes
-    from application of a key then that key should not have been used. The
+    from application of a key, then that key should not have been used. The
     exception, however, is that even if there are many collisions, if the
     first group is small and one does not need to process all items in the
     list then time will not be wasted sorting what one was not interested
@@ -665,8 +601,7 @@ def ordered(seq, keys=None, default=True, warn=False):
                 if len(u) > 1:
                     raise ValueError(
                         'not enough keys to break ties: %s' % u)
-        for v in d[k]:
-            yield v
+        yield from d[k]
         d.pop(k)
 
 # If HAS_GMPY is 0, no supported version of gmpy is available. Otherwise,
@@ -706,6 +641,8 @@ if GROUND_TYPES != 'python':
             module_version_attr='version', module_version_attr_call_args=())
         if gmpy:
             HAS_GMPY = 1
+else:
+    gmpy = None
 
 if GROUND_TYPES == 'auto':
     if HAS_GMPY:
@@ -719,191 +656,9 @@ if GROUND_TYPES == 'gmpy' and not HAS_GMPY:
     GROUND_TYPES = 'python'
 
 # SYMPY_INTS is a tuple containing the base types for valid integer types.
-SYMPY_INTS = integer_types
+SYMPY_INTS = (int, )  # type: Tuple[Type, ...]
 
 if GROUND_TYPES == 'gmpy':
     SYMPY_INTS += (type(gmpy.mpz(0)),)
 
-
-# lru_cache compatible with py2.7 copied directly from
-#   http://code.activestate.com/
-#   recipes/578078-py26-and-py30-backport-of-python-33s-lru-cache/
-from collections import namedtuple
-from functools import update_wrapper
-from threading import RLock
-
-_CacheInfo = namedtuple("CacheInfo", ["hits", "misses", "maxsize", "currsize"])
-
-class _HashedSeq(list):
-    __slots__ = 'hashvalue'
-
-    def __init__(self, tup, hash=hash):
-        self[:] = tup
-        self.hashvalue = hash(tup)
-
-    def __hash__(self):
-        return self.hashvalue
-
-def _make_key(args, kwds, typed,
-             kwd_mark = (object(),),
-             fasttypes = set((int, str, frozenset, type(None))),
-             sorted=sorted, tuple=tuple, type=type, len=len):
-    'Make a cache key from optionally typed positional and keyword arguments'
-    key = args
-    if kwds:
-        sorted_items = sorted(kwds.items())
-        key += kwd_mark
-        for item in sorted_items:
-            key += item
-    if typed:
-        key += tuple(type(v) for v in args)
-        if kwds:
-            key += tuple(type(v) for k, v in sorted_items)
-    elif len(key) == 1 and type(key[0]) in fasttypes:
-        return key[0]
-    return _HashedSeq(key)
-
-def lru_cache(maxsize=100, typed=False):
-    """Least-recently-used cache decorator.
-
-    If *maxsize* is set to None, the LRU features are disabled and the cache
-    can grow without bound.
-
-    If *typed* is True, arguments of different types will be cached separately.
-    For example, f(3.0) and f(3) will be treated as distinct calls with
-    distinct results.
-
-    Arguments to the cached function must be hashable.
-
-    View the cache statistics named tuple (hits, misses, maxsize, currsize) with
-    f.cache_info().  Clear the cache and statistics with f.cache_clear().
-    Access the underlying function with f.__wrapped__.
-
-    See:  http://en.wikipedia.org/wiki/Cache_algorithms#Least_Recently_Used
-
-    """
-
-    # Users should only access the lru_cache through its public API:
-    #       cache_info, cache_clear, and f.__wrapped__
-    # The internals of the lru_cache are encapsulated for thread safety and
-    # to allow the implementation to change (including a possible C version).
-
-    def decorating_function(user_function):
-
-        cache = dict()
-        stats = [0, 0]                  # make statistics updateable non-locally
-        HITS, MISSES = 0, 1             # names for the stats fields
-        make_key = _make_key
-        cache_get = cache.get           # bound method to lookup key or return None
-        _len = len                      # localize the global len() function
-        lock = RLock()                  # because linkedlist updates aren't threadsafe
-        root = []                       # root of the circular doubly linked list
-        root[:] = [root, root, None, None]      # initialize by pointing to self
-        nonlocal_root = [root]                  # make updateable non-locally
-        PREV, NEXT, KEY, RESULT = 0, 1, 2, 3    # names for the link fields
-
-        if maxsize == 0:
-
-            def wrapper(*args, **kwds):
-                # no caching, just do a statistics update after a successful call
-                result = user_function(*args, **kwds)
-                stats[MISSES] += 1
-                return result
-
-        elif maxsize is None:
-
-            def wrapper(*args, **kwds):
-                # simple caching without ordering or size limit
-                key = make_key(args, kwds, typed)
-                result = cache_get(key, root)   # root used here as a unique not-found sentinel
-                if result is not root:
-                    stats[HITS] += 1
-                    return result
-                result = user_function(*args, **kwds)
-                cache[key] = result
-                stats[MISSES] += 1
-                return result
-
-        else:
-
-            def wrapper(*args, **kwds):
-                # size limited caching that tracks accesses by recency
-                try:
-                    key = make_key(args, kwds, typed) if kwds or typed else args
-                except TypeError:
-                    stats[MISSES] += 1
-                    return user_function(*args, **kwds)
-                with lock:
-                    link = cache_get(key)
-                    if link is not None:
-                        # record recent use of the key by moving it to the front of the list
-                        root, = nonlocal_root
-                        link_prev, link_next, key, result = link
-                        link_prev[NEXT] = link_next
-                        link_next[PREV] = link_prev
-                        last = root[PREV]
-                        last[NEXT] = root[PREV] = link
-                        link[PREV] = last
-                        link[NEXT] = root
-                        stats[HITS] += 1
-                        return result
-                result = user_function(*args, **kwds)
-                with lock:
-                    root, = nonlocal_root
-                    if key in cache:
-                        # getting here means that this same key was added to the
-                        # cache while the lock was released.  since the link
-                        # update is already done, we need only return the
-                        # computed result and update the count of misses.
-                        pass
-                    elif _len(cache) >= maxsize:
-                        # use the old root to store the new key and result
-                        oldroot = root
-                        oldroot[KEY] = key
-                        oldroot[RESULT] = result
-                        # empty the oldest link and make it the new root
-                        root = nonlocal_root[0] = oldroot[NEXT]
-                        oldkey = root[KEY]
-                        oldvalue = root[RESULT]
-                        root[KEY] = root[RESULT] = None
-                        # now update the cache dictionary for the new links
-                        del cache[oldkey]
-                        cache[key] = oldroot
-                    else:
-                        # put result in a new link at the front of the list
-                        last = root[PREV]
-                        link = [last, root, key, result]
-                        last[NEXT] = root[PREV] = cache[key] = link
-                    stats[MISSES] += 1
-                return result
-
-        def cache_info():
-            """Report cache statistics"""
-            with lock:
-                return _CacheInfo(stats[HITS], stats[MISSES], maxsize, len(cache))
-
-        def cache_clear():
-            """Clear the cache and cache statistics"""
-            with lock:
-                cache.clear()
-                root = nonlocal_root[0]
-                root[:] = [root, root, None, None]
-                stats[:] = [0, 0]
-
-        wrapper.__wrapped__ = user_function
-        wrapper.cache_info = cache_info
-        wrapper.cache_clear = cache_clear
-        return update_wrapper(wrapper, user_function)
-
-    return decorating_function
-### End of backported lru_cache
-
-if sys.version_info[:2] >= (3, 3):
-    # 3.2 has an lru_cache with an incompatible API
-    from functools import lru_cache
-
-try:
-    from itertools import filterfalse
-except ImportError:  # Python 2.7
-    def filterfalse(pred, itr):
-        return filter(lambda x: not pred(x), itr)
+from time import perf_counter as clock
