@@ -1,4 +1,4 @@
-"""
+r"""
 This module is intended for solving recurrences or, in other words,
 difference equations. Currently supported are linear, inhomogeneous
 equations with polynomial or rational coefficients.
@@ -19,9 +19,9 @@ where `k > 0` and `a_{i}(n)` are polynomials in `n`. To use
 ``rsolve_X`` we need to put all coefficients in to a list ``L`` of
 `k+1` elements the following way:
 
-    ``L = [ a_{0}(n), ..., a_{k-1}(n), a_{k}(n) ]``
+    ``L = [a_{0}(n), ..., a_{k-1}(n), a_{k}(n)]``
 
-where ``L[i]``, for `i=0, \dots, k`, maps to
+where ``L[i]``, for `i=0, \ldots, k`, maps to
 `a_{i}(n) y(n+i)` (`y(n+i)` is implicit).
 
 For example if we would like to compute `m`-th Bernoulli polynomial
@@ -42,34 +42,32 @@ n**4 - 2*n**3 + n**2 - 1/30
 
 For the sake of completeness, `f(n)` can be:
 
-    [1] a polynomial              -> rsolve_poly
-    [2] a rational function       -> rsolve_ratio
+    [1] a polynomial               -> rsolve_poly
+    [2] a rational function        -> rsolve_ratio
     [3] a hypergeometric function  -> rsolve_hyper
 """
-from __future__ import print_function, division
-
 from collections import defaultdict
 
 from sympy.core.singleton import S
-from sympy.core.numbers import Rational
+from sympy.core.numbers import Rational, I
 from sympy.core.symbol import Symbol, Wild, Dummy
 from sympy.core.relational import Equality
 from sympy.core.add import Add
 from sympy.core.mul import Mul
 from sympy.core import sympify
 
-from sympy.simplify import simplify, hypersimp, hypersimilar
+from sympy.simplify import simplify, hypersimp, hypersimilar  # type: ignore
 from sympy.solvers import solve, solve_undetermined_coeffs
 from sympy.polys import Poly, quo, gcd, lcm, roots, resultant
 from sympy.functions import binomial, factorial, FallingFactorial, RisingFactorial
 from sympy.matrices import Matrix, casoratian
 from sympy.concrete import product
-from sympy.core.compatibility import default_sort_key, xrange
+from sympy.core.compatibility import default_sort_key
 from sympy.utilities.iterables import numbered_symbols
 
 
-def rsolve_poly(coeffs, f, n, **hints):
-    """
+def rsolve_poly(coeffs, f, n, shift=0, **hints):
+    r"""
     Given linear recurrence operator `\operatorname{L}` of order
     `k` with polynomial coefficients and inhomogeneous equation
     `\operatorname{L} y = f`, where `f` is a polynomial, we seek for
@@ -129,14 +127,14 @@ def rsolve_poly(coeffs, f, n, **hints):
 
     r = len(coeffs) - 1
 
-    coeffs = [ Poly(coeff, n) for coeff in coeffs ]
+    coeffs = [Poly(coeff, n) for coeff in coeffs]
 
-    polys = [ Poly(0, n) ] * (r + 1)
-    terms = [ (S.Zero, S.NegativeInfinity) ] *(r + 1)
+    polys = [Poly(0, n)]*(r + 1)
+    terms = [(S.Zero, S.NegativeInfinity)]*(r + 1)
 
-    for i in xrange(0, r + 1):
-        for j in xrange(i, r + 1):
-            polys[i] += coeffs[j]*binomial(j, i)
+    for i in range(r + 1):
+        for j in range(i, r + 1):
+            polys[i] += coeffs[j]*(binomial(j, i).as_poly(n))
 
         if not polys[i].is_zero:
             (exp,), coeff = polys[i].LT()
@@ -144,7 +142,7 @@ def rsolve_poly(coeffs, f, n, **hints):
 
     d = b = terms[0][1]
 
-    for i in xrange(1, r + 1):
+    for i in range(1, r + 1):
         if terms[i][1] > d:
             d = terms[i][1]
 
@@ -157,7 +155,7 @@ def rsolve_poly(coeffs, f, n, **hints):
 
     degree_poly = S.Zero
 
-    for i in xrange(0, r + 1):
+    for i in range(r + 1):
         if terms[i][1] - i == b:
             degree_poly += terms[i][0]*FallingFactorial(x, i)
 
@@ -189,17 +187,17 @@ def rsolve_poly(coeffs, f, n, **hints):
         C = []
         y = E = S.Zero
 
-        for i in xrange(0, N + 1):
-            C.append(Symbol('C' + str(i)))
+        for i in range(N + 1):
+            C.append(Symbol('C' + str(i + shift)))
             y += C[i] * n**i
 
-        for i in xrange(0, r + 1):
+        for i in range(r + 1):
             E += coeffs[i].as_expr()*y.subs(n, n + i)
 
         solutions = solve_undetermined_coeffs(E - f, C, n)
 
         if solutions is not None:
-            C = [ c for c in C if (c not in solutions) ]
+            C = [c for c in C if (c not in solutions)]
             result = y.subs(solutions)
         else:
             return None  # TBD
@@ -225,24 +223,24 @@ def rsolve_poly(coeffs, f, n, **hints):
             B = S.One
             D = p.subs(n, a + k)
 
-            for i in xrange(1, k + 1):
-                B *= -Rational(k - i + 1, i)
+            for i in range(1, k + 1):
+                B *= Rational(i - k - 1, i)
                 D += B * p.subs(n, a + k - i)
 
             return D
 
         alpha = {}
 
-        for i in xrange(-A, d + 1):
+        for i in range(-A, d + 1):
             I = _one_vector(d + 1)
 
-            for k in xrange(1, d + 1):
+            for k in range(1, d + 1):
                 I[k] = I[k - 1] * (x + i - k + 1)/k
 
             alpha[i] = S.Zero
 
-            for j in xrange(0, A + 1):
-                for k in xrange(0, d + 1):
+            for j in range(A + 1):
+                for k in range(d + 1):
                     B = binomial(k, i + j)
                     D = _delta(polys[j].as_expr(), k)
 
@@ -251,66 +249,66 @@ def rsolve_poly(coeffs, f, n, **hints):
         V = Matrix(U, A, lambda i, j: int(i == j))
 
         if homogeneous:
-            for i in xrange(A, U):
+            for i in range(A, U):
                 v = _zero_vector(A)
 
-                for k in xrange(1, A + b + 1):
+                for k in range(1, A + b + 1):
                     if i - k < 0:
                         break
 
                     B = alpha[k - A].subs(x, i - k)
 
-                    for j in xrange(0, A):
+                    for j in range(A):
                         v[j] += B * V[i - k, j]
 
                 denom = alpha[-A].subs(x, i)
 
-                for j in xrange(0, A):
+                for j in range(A):
                     V[i, j] = -v[j] / denom
         else:
             G = _zero_vector(U)
 
-            for i in xrange(A, U):
+            for i in range(A, U):
                 v = _zero_vector(A)
                 g = S.Zero
 
-                for k in xrange(1, A + b + 1):
+                for k in range(1, A + b + 1):
                     if i - k < 0:
                         break
 
                     B = alpha[k - A].subs(x, i - k)
 
-                    for j in xrange(0, A):
+                    for j in range(A):
                         v[j] += B * V[i - k, j]
 
                     g += B * G[i - k]
 
                 denom = alpha[-A].subs(x, i)
 
-                for j in xrange(0, A):
+                for j in range(A):
                     V[i, j] = -v[j] / denom
 
                 G[i] = (_delta(f, i - A) - g) / denom
 
         P, Q = _one_vector(U), _zero_vector(A)
 
-        for i in xrange(1, U):
+        for i in range(1, U):
             P[i] = (P[i - 1] * (n - a - i + 1)/i).expand()
 
-        for i in xrange(0, A):
-            Q[i] = Add(*[ (v*p).expand() for v, p in zip(V[:, i], P) ])
+        for i in range(A):
+            Q[i] = Add(*[(v*p).expand() for v, p in zip(V[:, i], P)])
 
         if not homogeneous:
-            h = Add(*[ (g*p).expand() for g, p in zip(G, P) ])
+            h = Add(*[(g*p).expand() for g, p in zip(G, P)])
 
-        C = [ Symbol('C' + str(i)) for i in xrange(0, A) ]
+        C = [Symbol('C' + str(i + shift)) for i in range(A)]
 
-        g = lambda i: Add(*[ c*_delta(q, i) for c, q in zip(C, Q) ])
+        g = lambda i: Add(*[c*_delta(q, i) for c, q in zip(C, Q)])
 
         if homogeneous:
-            E = [ g(i) for i in xrange(N + 1, U) ]
+            E = [g(i) for i in range(N + 1, U)]
         else:
-            E = [ g(i) + _delta(h, i) for i in xrange(N + 1, U) ]
+            E = [g(i) + _delta(h, i) for i in range(N + 1, U)]
 
         if E != []:
             solutions = solve(E, *C)
@@ -347,7 +345,7 @@ def rsolve_poly(coeffs, f, n, **hints):
 
 
 def rsolve_ratio(coeffs, f, n, **hints):
-    """
+    r"""
     Given linear recurrence operator `\operatorname{L}` of order `k`
     with polynomial coefficients and inhomogeneous equation
     `\operatorname{L} y = f`, where `f` is a polynomial, we seek
@@ -427,23 +425,23 @@ def rsolve_ratio(coeffs, f, n, **hints):
     else:
         C, numers = S.One, [S.Zero]*(r + 1)
 
-        for i in xrange(int(max(nni_roots)), -1, -1):
+        for i in range(int(max(nni_roots)), -1, -1):
             d = gcd(A, B.subs(n, n + i), n)
 
             A = quo(A, d, n)
             B = quo(B, d.subs(n, n - i), n)
 
-            C *= Mul(*[ d.subs(n, n - j) for j in xrange(0, i + 1) ])
+            C *= Mul(*[d.subs(n, n - j) for j in range(i + 1)])
 
-        denoms = [ C.subs(n, n + i) for i in range(0, r + 1) ]
+        denoms = [C.subs(n, n + i) for i in range(r + 1)]
 
-        for i in range(0, r + 1):
+        for i in range(r + 1):
             g = gcd(coeffs[i], denoms[i], n)
 
             numers[i] = quo(coeffs[i], g, n)
             denoms[i] = quo(denoms[i], g, n)
 
-        for i in xrange(0, r + 1):
+        for i in range(r + 1):
             numers[i] *= Mul(*(denoms[:i] + denoms[i + 1:]))
 
         result = rsolve_poly(numers, f * Mul(*denoms), n, **hints)
@@ -458,7 +456,7 @@ def rsolve_ratio(coeffs, f, n, **hints):
 
 
 def rsolve_hyper(coeffs, f, n, **hints):
-    """
+    r"""
     Given linear recurrence operator `\operatorname{L}` of order `k`
     with polynomial coefficients and inhomogeneous equation
     `\operatorname{L} y = f` we seek for all hypergeometric solutions
@@ -498,7 +496,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
     >>> from sympy.abc import x
 
     >>> rsolve_hyper([-1, -1, 1], 0, x)
-    C0*(1/2 + sqrt(5)/2)**x + C1*(-sqrt(5)/2 + 1/2)**x
+    C0*(1/2 - sqrt(5)/2)**x + C1*(1/2 + sqrt(5)/2)**x
 
     >>> rsolve_hyper([-1, 1], 1 + x, x)
     C0 + x*(x + 1)/2
@@ -544,11 +542,11 @@ def rsolve_hyper(coeffs, f, n, **hints):
 
         for i, g in enumerate(inhomogeneous):
             coeff, polys = S.One, coeffs[:]
-            denoms = [ S.One ] * (r + 1)
+            denoms = [S.One]*(r + 1)
 
             s = hypersimp(g, n)
 
-            for j in xrange(1, r + 1):
+            for j in range(1, r + 1):
                 coeff *= s.subs(n, n + j - 1)
 
                 p, q = coeff.as_numer_denom()
@@ -556,7 +554,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
                 polys[j] *= p
                 denoms[j] = q
 
-            for j in xrange(0, r + 1):
+            for j in range(r + 1):
                 polys[j] *= Mul(*(denoms[:j] + denoms[j + 1:]))
 
             R = rsolve_poly(polys, Mul(*denoms), n)
@@ -574,10 +572,10 @@ def rsolve_hyper(coeffs, f, n, **hints):
 
     p, q = coeffs[0], coeffs[r].subs(n, n - r + 1)
 
-    p_factors = [ z for z in roots(p, n).keys() ]
-    q_factors = [ z for z in roots(q, n).keys() ]
+    p_factors = [z for z in roots(p, n).keys()]
+    q_factors = [z for z in roots(q, n).keys()]
 
-    factors = [ (S.One, S.One) ]
+    factors = [(S.One, S.One)]
 
     for p in p_factors:
         for q in q_factors:
@@ -586,8 +584,8 @@ def rsolve_hyper(coeffs, f, n, **hints):
             else:
                 factors += [(n - p, n - q)]
 
-    p = [ (n - p, S.One) for p in p_factors ]
-    q = [ (S.One, n - q) for q in q_factors ]
+    p = [(n - p, S.One) for p in p_factors]
+    q = [(S.One, n - q) for q in q_factors]
 
     factors = p + factors + q
 
@@ -595,9 +593,9 @@ def rsolve_hyper(coeffs, f, n, **hints):
         polys, degrees = [], []
         D = A*B.subs(n, n + r - 1)
 
-        for i in xrange(0, r + 1):
-            a = Mul(*[ A.subs(n, n + j) for j in xrange(0, i) ])
-            b = Mul(*[ B.subs(n, n + j) for j in xrange(i, r) ])
+        for i in range(r + 1):
+            a = Mul(*[A.subs(n, n + j) for j in range(i)])
+            b = Mul(*[B.subs(n, n + j) for j in range(i, r)])
 
             poly = quo(coeffs[i]*a*b, D, n)
             polys.append(poly.as_poly(n))
@@ -605,9 +603,12 @@ def rsolve_hyper(coeffs, f, n, **hints):
             if not poly.is_zero:
                 degrees.append(polys[i].degree())
 
-        d, poly = max(degrees), S.Zero
+        if degrees:
+            d, poly = max(degrees), S.Zero
+        else:
+            return None
 
-        for i in xrange(0, r + 1):
+        for i in range(r + 1):
             coeff = polys[i].nth(d)
 
             if coeff is not S.Zero:
@@ -617,7 +618,14 @@ def rsolve_hyper(coeffs, f, n, **hints):
             if z.is_zero:
                 continue
 
-            (C, s) = rsolve_poly([ polys[i]*z**i for i in xrange(r + 1) ], 0, n, symbols=True)
+            recurr_coeffs = [polys[i].as_expr()*z**i for i in range(r + 1)]
+            if d == 0 and 0 != Add(*[recurr_coeffs[j]*j for j in range(1, r + 1)]):
+                # faster inline check (than calling rsolve_poly) for a
+                # constant solution to a constant coefficient recurrence.
+                C = Symbol("C" + str(len(symbols)))
+                s = [C]
+            else:
+                C, s = rsolve_poly(recurr_coeffs, 0, n, len(symbols), symbols=True)
 
             if C is not None and C is not S.Zero:
                 symbols |= set(s)
@@ -629,7 +637,9 @@ def rsolve_hyper(coeffs, f, n, **hints):
                 # start the product with the term y(n_root + 1).
                 n0 = 0
                 for n_root in roots(ratio.as_numer_denom()[1], n).keys():
-                    if (n0 < (n_root + 1)) == True:
+                    if n_root.has(I):
+                        return None
+                    elif (n0 < (n_root + 1)) == True:
                         n0 = n_root + 1
                 K = product(ratio, (n, n0, n - 1))
                 if K.has(factorial, FallingFactorial, RisingFactorial):
@@ -648,37 +658,38 @@ def rsolve_hyper(coeffs, f, n, **hints):
         return None
 
     if hints.get('symbols', False):
-        symbols |= set([s for s, k in sk])
+        # XXX: This returns the symbols in a non-deterministic order
+        symbols |= {s for s, k in sk}
         return (result, list(symbols))
     else:
         return result
 
 
 def rsolve(f, y, init=None):
-    """
+    r"""
     Solve univariate recurrence with rational coefficients.
 
     Given `k`-th order linear recurrence `\operatorname{L} y = f`,
     or equivalently:
 
     .. math:: a_{k}(n) y(n+k) + a_{k-1}(n) y(n+k-1) +
-              \dots + a_{0}(n) y(n) = f(n)
+              \cdots + a_{0}(n) y(n) = f(n)
 
-    where `a_{i}(n)`, for `i=0, \dots, k`, are polynomials or rational
+    where `a_{i}(n)`, for `i=0, \ldots, k`, are polynomials or rational
     functions in `n`, and `f` is a hypergeometric function or a sum
     of a fixed number of pairwise dissimilar hypergeometric terms in
     `n`, finds all solutions or returns ``None``, if none were found.
 
     Initial conditions can be given as a dictionary in two forms:
 
-        (1) ``{   n_0  : v_0,   n_1  : v_1, ...,   n_m  : v_m }``
-        (2) ``{ y(n_0) : v_0, y(n_1) : v_1, ..., y(n_m) : v_m }``
+        (1) ``{  n_0  : v_0,   n_1  : v_1, ...,   n_m  : v_m}``
+        (2) ``{y(n_0) : v_0, y(n_1) : v_1, ..., y(n_m) : v_m}``
 
     or as a list ``L`` of values:
 
-        ``L = [ v_0, v_1, ..., v_m ]``
+        ``L = [v_0, v_1, ..., v_m]``
 
-    where ``L[i] = v_i``, for `i=0, \dots, m`, maps to `y(n_i)`.
+    where ``L[i] = v_i``, for `i=0, \ldots, m`, maps to `y(n_i)`.
 
     Examples
     ========
@@ -697,7 +708,7 @@ def rsolve(f, y, init=None):
     >>> rsolve(f, y(n))
     2**n*C0 + C1*factorial(n)
 
-    >>> rsolve(f, y(n), { y(0):0, y(1):3 })
+    >>> rsolve(f, y(n), {y(0):0, y(1):3})
     3*2**n - 3*factorial(n)
 
     See Also
@@ -716,37 +727,35 @@ def rsolve(f, y, init=None):
     # y(n) + a*(y(n + 1) + y(n - 1))/2
     f = f.expand().collect(y.func(Wild('m', integer=True)))
 
-    h_part = defaultdict(lambda: S.Zero)
-    i_part = S.Zero
+    h_part = defaultdict(list)
+    i_part = []
     for g in Add.make_args(f):
-        coeff = S.One
-        kspec = None
-        for h in Mul.make_args(g):
-            if h.is_Function:
-                if h.func == y.func:
-                    result = h.args[0].match(n + k)
-
-                    if result is not None:
-                        kspec = int(result[k])
-                    else:
-                        raise ValueError(
-                            "'%s(%s+k)' expected, got '%s'" % (y.func, n, h))
-                else:
-                    raise ValueError(
-                        "'%s' expected, got '%s'" % (y.func, h.func))
-            else:
-                coeff *= h
-
-        if kspec is not None:
-            h_part[kspec] += coeff
-        else:
-            i_part += coeff
+        coeff, dep = g.as_coeff_mul(y.func)
+        if not dep:
+            i_part.append(coeff)
+            continue
+        for h in dep:
+            if h.is_Function and h.func == y.func:
+                result = h.args[0].match(n + k)
+                if result is not None:
+                    h_part[int(result[k])].append(coeff)
+                    continue
+            raise ValueError(
+                "'%s(%s + k)' expected, got '%s'" % (y.func, n, h))
+    for k in h_part:
+        h_part[k] = Add(*h_part[k])
+    h_part.default_factory = lambda: 0
+    i_part = Add(*i_part)
 
     for k, coeff in h_part.items():
         h_part[k] = simplify(coeff)
 
     common = S.One
 
+    if not i_part.is_zero and not i_part.is_hypergeometric(n) and \
+       not (i_part.is_Add and all(map(lambda x: x.is_hypergeometric(n), i_part.expand().args))):
+        raise ValueError("The independent term should be a sum of hypergeometric functions, got '%s'" % i_part)
+
     for coeff in h_part.values():
         if coeff.is_rational_function(n):
             if not coeff.is_polynomial(n):
@@ -782,7 +791,7 @@ def rsolve(f, y, init=None):
         H_part = h_part
 
     K_max = max(H_part.keys())
-    coeffs = [H_part[i] for i in xrange(K_max + 1)]
+    coeffs = [H_part[i] for i in range(K_max + 1)]
 
     result = rsolve_hyper(coeffs, -i_part, n, symbols=True)
 
@@ -795,8 +804,8 @@ def rsolve(f, y, init=None):
         init = None
 
     if symbols and init is not None:
-        if type(init) is list:
-            init = dict([(i, init[i]) for i in xrange(len(init))])
+        if isinstance(init, list):
+            init = {i: init[i] for i in range(len(init))}
 
         equations = []
 
@@ -808,10 +817,10 @@ def rsolve(f, y, init=None):
                     i = int(k.args[0])
                 else:
                     raise ValueError("Integer or term expected, got '%s'" % k)
-            try:
+
+            eq = solution.subs(n, i) - v
+            if eq.has(S.NaN):
                 eq = solution.limit(n, i) - v
-            except NotImplementedError:
-                eq = solution.subs(n, i) - v
             equations.append(eq)
 
         result = solve(equations, *symbols)
